-- top_level.vhd : prenos dat do pinheaderu
-- Copyright (C) 2013 Brno University of Technology,
--                    Faculty of Information Technology
-- Author(s): Lucie Matusova   xmatus21@stud.fit.vutbr.cz
--

library IEEE;
    use IEEE.std_logic_1164.ALL;
    use IEEE.std_logic_ARITH.ALL;
    use IEEE.std_logic_UNSIGNED.ALL;

architecture main of tlv_gp_ifc is

   signal addr_out : std_logic_vector (5 downto 0);
   signal data_out : std_logic_vector (7 downto 0);

   -- senzoric data from the robot
   signal data   : std_logic_vector (7 downto 0);
   signal sel    : std_logic_vector (7 downto 0);
   signal wen    : std_logic;
   signal enable : std_logic_vector (9 downto 0); -- number of signals coming from the robot

   -- display
   signal dis_addr     : std_logic_vector(0 downto 0);
   signal dis_data_out : std_logic_vector(15 downto 0);
   signal dis_write_en : std_logic;

   -----------------------------------------------------------------------------

   component SPI_adc
      generic (
         ADDR_WIDTH : integer;   -- the whole addr width
         DATA_WIDTH : integer;   -- number of bits to receive
         ADDR_OUT_WIDTH : integer;
         BASE_ADDR  : integer
      );
      port (
         CLK      : in  std_logic;

         CS       : in  std_logic;
         DO       : in  std_logic;
         DO_VLD   : in  std_logic;
         DI       : out std_logic;
         DI_REQ   : in  std_logic;

         ADDR     : out  std_logic_vector (ADDR_OUT_WIDTH-1 downto 0);
         DATA_OUT : out  std_logic_vector (DATA_WIDTH-1 downto 0);
         DATA_IN  : in   std_logic_vector (DATA_WIDTH-1 downto 0);

         WRITE_EN : out  std_logic;
         READ_EN  : out  std_logic
      );
   end component;

   component decoder
      generic (
         ADDR_OUT_WIDTH : integer;
         EN_WIDTH       : integer
      );
      port (
         I   : in std_logic_vector(ADDR_OUT_WIDTH-1 downto 0);
         EN  : out std_logic_vector(EN_WIDTH-1 downto 0)  
      );
   end component;

   component lcd_raw_controller
      port (
         RST      : in std_logic;
         CLK      : in std_logic;

         DATA_IN  : in std_logic_vector (15 downto 0);
         WRITE_EN : in std_logic;

         DISPLAY_RS   : out   std_logic;
         DISPLAY_DATA : inout std_logic_vector(7 downto 0);
         DISPLAY_RW   : out   std_logic;
         DISPLAY_EN   : out   std_logic
      );
   end component;

-- ------------------------------------------------------------------------------ 

begin

   SPI_adc_robot: SPI_adc
      generic map(
         ADDR_WIDTH => 8,       
         DATA_WIDTH => 8,       
         ADDR_OUT_WIDTH => 6,
         BASE_ADDR  => 16#80# -- address space from 0x80
                              -- 0x80 (1000 0000), 0x80 - 0x8F 
      )
      port map(
         CLK   	  =>  CLK,
   
         CS       => SPI_CS,
         DO       => SPI_DO,
         DO_VLD   => SPI_DO_VLD,
         DI       => SPI_DI,
         DI_REQ   => SPI_DI_REQ,
   
         ADDR     => addr_out,
         DATA_OUT => data_out,
         DATA_IN  => "11111111",
         WRITE_EN => wen,
         READ_EN  => open
      );

   -- address decoder
   dec : decoder
      generic map(
         ADDR_OUT_WIDTH => 6,
         EN_WIDTH => 10
      )
      port map(
         I  => addr_out,
         EN => enable
      );
  
  -- -----------------------------------------
  -- register for senzoric data
  -- X(20..18....8..6) -----> 25, .., 11 (FPGA)
  process (CLK)
  begin
    if (CLK='1') and (CLK'event) then
      if (RESET='1') then
         data <= (others => '0');
      elsif (enable(1)='1' and wen='1') then
         data <= data_out;
      end if;
    end if; 
  end process;

  X(6)  <= data(0);
  X(8)  <= data(1);
  X(10) <= data(2);
  X(12) <= data(3);
  X(14) <= data(4);
  X(16) <= data(5);
  X(18) <= data(6);
  X(20) <= data(7);
  
  -- -----------------------------------------
  -- register for sel - select
  -- X(21..19....9..7) -----> 26, .., 12 (FPGA)
  process (CLK)
  begin
    if (CLK='1') and (CLK'event) then
      if (RESET='1') then
        sel <= (others => '0');
      elsif (enable(2)='1' and wen='1') then
	      sel <= data_out;
        sel(4) <= '1';     -- senzor addr and senzor data now valid
      else
        sel(4) <= '0';     -- senzor addr and senzor data not valid anymore
      end if;
    end if; 
  end process;
 
  X(7)  <= sel(0);
  X(9)  <= sel(1);
  X(11) <= sel(2);
  X(13) <= sel(3);
  X(15) <= sel(4);
  X(17) <= sel(5);
  X(19) <= sel(6);
  X(21) <= sel(7);
  
  -- -----------------------------------------
  -- SPI decoder - display
  spidecd: SPI_adc
	generic map (
            ADDR_WIDTH => 8,     -- sirka adresy 8 bitu
            DATA_WIDTH => 16,    -- sirka dat 16 bitu
            ADDR_OUT_WIDTH => 1, -- sirka adresy na vystupu 1 bit
            BASE_ADDR  => 16#00# -- adresovy prostor od 0x00-0x01
         )
         port map (
            CLK      => CLK,
            CS       => SPI_CS,

            DO       => SPI_DO,
            DO_VLD   => SPI_DO_VLD,
            DI       => SPI_DI,
            DI_REQ   => SPI_DI_REQ,

            ADDR     => dis_addr,
            DATA_OUT => dis_data_out,
            DATA_IN  => "0000000000000000",
            WRITE_EN => dis_write_en,
            READ_EN  => open
         );

   -- LCD display controller
   lcdctrl: lcd_raw_controller
         port map (
            RST    =>  RESET,
            CLK    =>  CLK,

            DATA_IN  => dis_data_out,
            WRITE_EN => dis_write_en,

            -- display signals
            DISPLAY_RS   => LRS,
            DISPLAY_DATA => LD,
            DISPLAY_RW   => LRW,
            DISPLAY_EN   => LE
         );

end main;
